SystemOrganization addCategory: #'EjerciciosPoo-TusLibros'!

Object subclass: #Cart
	instanceVariableNames: 'contents catalog'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Cart class methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 23:04'!
withCatalog: aDictionary 
	^Cart new initializeCatalogOn: aDictionary! !

!Cart class methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 23:05'!
withPairedCollection: aCollection
	^Cart withCatalog:  (Dictionary newFromPairs: aCollection)! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 23:55'!
add: anISBN withQuantity: aNumber
	
	aNumber > 0 ifFalse: [Error signal: 'Not a valid quantity'].
	(self isAListedISBN: anISBN) ifFalse: [Error signal: 'Book not in catalog'].
	contents add: anISBN withOccurrences: aNumber.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 23:21'!
calculateTotalPrice
	| accumulated |
	accumulated := 0.
	contents do: [ :each | accumulated := accumulated + (catalog at: each) ].
	^accumulated ! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/13/2012 21:29'!
initializeCatalogOn: aCollection 
	catalog := aCollection.
	contents := Bag new.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 23:08'!
isAListedISBN: anISBN  
	
	^catalog includesKey: anISBN.! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/13/2012 21:32'!
isEmpty
	^contents isEmpty! !

!Cart methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/12/2012 19:11'!
listContents
	^ contents.! !

Object subclass: #Cashier
	instanceVariableNames: 'calledMP'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!Cashier methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 23:52'!
callMPWithCC: aCreditCard forTotal: aNumber 
	calledMP := true.
	
	"TODO: Actually call MP"! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 23:27'!
checkCartIsNotEmpty: aCart
	aCart isEmpty
		ifTrue: [ Error signal: 'Cart should not be empty' ]! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 23:32'!
checkout: aCart withCreditCard: aCreditCard
	|total|
	
	self checkCartIsNotEmpty: aCart.
	self validateCreditCard: aCreditCard.
	
	total := aCart calculateTotalPrice.
	
	self callMPWithCC: aCreditCard forTotal: total! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 23:29'!
hasCalledMP
	^calledMP ! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 23:28'!
initialize
	calledMP := false! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 23:27'!
today
	"Este mtodo lo pusimos en cashier porque tuvimos problemas al agregarlo al paquete credit card, no lo suba al comitear"

	| currentDate |
	currentDate := Date current.
	^ MonthOfYear onMonthNumber: currentDate month index of: currentDate year! !

!Cashier methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 23:34'!
validateCreditCard: aCreditCard 
	(aCreditCard isExpiredOn: self today) ifTrue:[Error signal: 'Credit card is expired']. 
	(aCreditCard ownerName size > 30) ifTrue:[Error signal: 'Owner name is too long']. ! !

TestCase subclass: #TusLibrosTest
	instanceVariableNames: 'marketplace'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'EjerciciosPoo-TusLibros'!

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 21:48'!
creditCard
	^ CreditCard ownedBy: 'Juan' identifiedBy: 123456 expiresOn: (MonthOfYear januaryOf: Date current year + 1)! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 23:06'!
emptyCart
	
	 ^Cart withPairedCollection:  #('book1' 10  'book2' 5 'book3' 1)! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 22:26'!
expiredDate
	^MonthOfYear januaryOf: Date current year - 1! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 21:57'!
fullCart
	^ self emptyCart
		add: 'book1' withQuantity: 2;
		add: 'book2' withQuantity: 1! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 21:50'!
test01CartsAreCreatedEmpty
	self assert: (self emptyCart isEmpty)! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 23:55'!
test02CantAddBookNotInCatalogToCart
	| cart |
	cart := self emptyCart.
	self should: [ cart add: 'notBook' withQuantity: 1 ] raise: Error
		withExceptionDo: [ :anError | anError messageText = 'Book not in catalog' ]..
	self assert: cart isEmpty.! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 23:56'!
test03CantAddZeroOrNegativeNumberOfBooks
	| cart |
	cart := self emptyCart.
	self should: [ cart add: 'book1' withQuantity: 0 ] raise: Error
		withExceptionDo: [ :anError | anError messageText = 'Not a valid quantity' ].
	self assert: cart listContents isEmpty.
	self should: [ cart add: 'book1' withQuantity: -1 ] raise: Error
		withExceptionDo: [ :anError | anError messageText = 'Not a valid quantity' ].
	self assert: cart listContents isEmpty! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 21:50'!
test04ACanAddBookInCatalogue
	| cart list |
	cart := self emptyCart.
	cart add: 'book1' withQuantity: 1.
	list := cart listContents.
	self assert: (list occurrencesOf: 'book1') = 1! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 21:50'!
test04BCanAddMultipleBooksMultipleTimes
	| cart list |
	cart := self emptyCart.
	cart add: 'book1' withQuantity: 1.
	cart add: 'book1' withQuantity: 3.
	cart add: 'book2' withQuantity: 2.
	list := cart listContents.
	self assert: (list occurrencesOf: 'book1') = 4.
	self assert: (list occurrencesOf: 'book2') = 2! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 23:20'!
test05AEmptyCartCostsZero
	| cart list |
	cart := self emptyCart.
	self assert: cart calculateTotalPrice = 0! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 23:21'!
test05BCanAddMultipleBooksMultipleTimes
	| cart list |
	cart := self emptyCart.
	cart add: 'book1' withQuantity: 4.
	cart add: 'book2' withQuantity: 2.
	
	self assert: cart calculateTotalPrice = 50! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 23:30'!
test06CantCheckoutEmptyCart
	| cashier |
	cashier := Cashier new.
	self
		should: [ cashier checkout: self emptyCart withCreditCard: self creditCard ]
		raise: Error
		withExceptionDo: [ :anError | anError messageText = 'Cannot checkout empty cart' ].
	self deny: cashier hasCalledMP! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 23:30'!
test07CantCheckoutWithExpiredCreditCard
	| cashier |
	cashier := Cashier new.
	self
		should: [ 
			Cashier
				checkout: self fullCart
				withCreditCard: (CreditCard ownedBy: 'Juan' identifiedBy: 1222 expiresOn: self expiredDate) ]
		raise: Error
		withExceptionDo: [ :anError | anError messageText = 'Credit card is expired' ].
		self deny: cashier hasCalledMP! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 23:31'!
test08CantCheckoutWithNameTooLong
	| cashier |
	cashier := Cashier new.
	self
		should: [ 
			Cashier
				checkout: self fullCart
				withCreditCard:
					(CreditCard ownedBy: 'JuanJuanJuanJuanJuanJuanJuanJuan' identifiedBy: 1222 expiresOn: self expiredDate) ]
		raise: Error
		withExceptionDo: [ :anError | anError messageText = 'Owner name is too long' ].
	self deny: cashier hasCalledMP! !

!TusLibrosTest methodsFor: 'tests' stamp: 'RavasiAronson 6/17/2012 23:34'!
test09CanCheckoutWithValidCCAndCart
	| cashier |
	cashier := Cashier new.
	cashier 		checkout: self fullCart
				withCreditCard:
					(CreditCard ownedBy: 'Juan' identifiedBy: 1222 expiresOn: self validDate).

	self assert:  cashier hasCalledMP! !

!TusLibrosTest methodsFor: 'as yet unclassified' stamp: 'RavasiAronson 6/17/2012 23:34'!
validDate
	^MonthOfYear januaryOf: Date current year + 1! !
